;*************************************************************************
; Copyright (c) 2002 The University of Chicago, as Operator of Argonne
; National Laboratory.
; Copyright (c) 2002 The Regents of the University of California, as
; Operator of Los Alamos National Laboratory.
; This file is distributed subject to a Software License Agreement found
; in the file LICENSE that is included with this distribution. 
;*************************************************************************
;*****************************************************************************
pro sdds_table::startTable, numberExpectedRows
;+
; NAME:
;       SDDS_TABLE::STARTTABLE
;
; PURPOSE:
;       Prepare to place data in an SDDS table.
;
; CATEGORY:
;       SDDS interface.
;
; CALLING SEQUENCE:
;      startTable, numberExpecterRows
;
; INPUTS:
;	Number of Expected Rows for SDDS to preallocate memory
;
; OUTPUTS:
;       An exception will be generated on error and ERR_STRING set.
;
;
; EXAMPLE:
;	catch, error_status
;	if(error_status NE 0) then begin
;		print, 'ERROR_MESSAGE:',!ERR_STRING
;		return
;	endif
;       obj->startTable, numberExpectedRows
;
; MODIFICATION HISTORY:
;       Written by:     Robert Daly April 27,1999
;-
	errMsg=bytarr(120)
	i=0l
	numberRows=0l
	if( self.mode NE 'WRITE') then MESSAGE, ' File ' + self.file_name $
		+ ' not opened for write'
	if( self.free  NE 1) then MESSAGE, ' File ' + self.file_name $
		+ ' not initialized'

	if( self.layoutWritten  NE 1) then begin
		if(call_external('libsddsIDL.so', $
					'writeLayout',self.fd,errMsg)  EQ 0) then $
					MESSAGE, string(errmsg) + !ERR_STRING
		self.layoutWritten  = 1
	end
	if(n_elements(numberExpectedRows) EQ 0) then numberExpectedRows = 100
	numberRows=numberExpectedRows
	
	if(call_external('libsddsIDL.so', $
					'startTable',self.fd,numberRows,errMsg)  EQ 0 ) then $
					MESSAGE, string(errmsg) + !ERR_STRING
	self.pageStarted = 1
	
	return
end
;*****************************************************************************
pro sdds_table::writeTable
;+
; NAME:
;       SDDS_TABLE::WRITETABLE
;
; PURPOSE:
;       Place data in an SDDS table.
;
; CATEGORY:
;       SDDS interface.
;
; CALLING SEQUENCE:
;      writeTable
;
; INPUTS:
;	None
;
; OUTPUTS:
;       An exception will be generated on error and ERR_STRING set.
;
;
; EXAMPLE:
;	catch, error_status
;	if(error_status NE 0) then begin
;		print, 'ERROR_MESSAGE:',!ERR_STRING
;		return
;	endif
;       obj->writeTable
;
; MODIFICATION HISTORY:
;       Written by:     Robert Daly April 27,1999
;-
	errMsg=bytarr(120)

	if( self.mode NE 'WRITE') then MESSAGE, ' File ' + self.file_name $
		+ ' not opened for write'
	if( self.free  NE 1) then MESSAGE, ' File ' + self.file_name $
		+ ' not initialized'

	if(call_external('libsddsIDL.so', $
					'writeTable',self.fd,errMsg) EQ 0)  then $
					MESSAGE, string(errmsg) + !ERR_STRING
	return
end
;*****************************************************************************
pro sdds_table::terminate
;+
; NAME:
;       SDDS_TABLE::TERMINATE
;
; PURPOSE:
;       Erased table descriptions and frees all memory and closes file.
;
; CATEGORY:
;       SDDS interface.
;
; CALLING SEQUENCE:
;      terminate
;
; INPUTS:
;	None
;
; OUTPUTS:
;       An exception will be generated on error and ERR_STRING set.
;
;
; EXAMPLE:
;	catch, error_status
;	if(error_status NE 0) then begin
;		print, 'ERROR_MESSAGE:',!ERR_STRING
;		return
;	endif
;       obj->terminate
;
; MODIFICATION HISTORY:
;       Written by:     Robert Daly April 27,1999
;-
	errMsg=bytarr(120)
	if(call_external('libsddsIDL.so', $
					'terminate',self.fd,errMsg)  EQ 0)  then $
					MESSAGE, string(errmsg) + !ERR_STRING
	return
end
;*****************************************************************************
pro sdds_table::setParameterDefinition, aDefinition
;+
; NAME:
;       SDDS_TABLE::SETPARAMETERDEFINITION
;
; PURPOSE:
;       Set the description of a parameter for SDDS table.
;
; CATEGORY:
;       SDDS interface.
;
; CALLING SEQUENCE:
;      setParameterDefinition, aDefinition
;
; INPUTS:
;	aDefinition structure or array of structures)
;
; OUTPUTS:
;       An exception will be generated on error and ERR_STRING set.
;
;
; EXAMPLE:
;	aDefinition={parameterDefinition,NAME:'',SYMBOL:'',UNITS:'',DESCRIPTION:'', $
;		     FORMAT_STRING:'',FIXED_VALUE:'',TYPE:''}
;	catch, error_status
;	if(error_status NE 0) then begin
;		print, 'ERROR_MESSAGE:',!ERR_STRING
;		return
;	endif
;       setParameterDefinition, aDefinition
;
; MODIFICATION HISTORY:
;       Written by:     Robert Daly April 27,1999
;-
	errMsg=bytarr(120)
	i=0l
	if( self.mode NE 'WRITE') then MESSAGE, ' File ' + self.file_name $
		+ ' not opened for write'
	if( self.free  NE 1) then MESSAGE, ' File ' + self.file_name $
		+ ' not initialized'
	if n_elements(aDefinition) EQ  0 then MESSAGE, 'Must define at least one parameter'
		
	for i=0, i<n_elements(aDefinition) do begin
		if( (where(tag_names(aDefinition[i]) EQ 'NAME'))[0] EQ  -1)  then $
			MESSAGE, 'NAME must be specified in definition'
		aNAME=aDefinition.NAME
		aTYPE=1l
		if( (where(tag_names(aDefinition[i]) EQ 'TYPE'))[0] NE -1) then begin
			case 	aDefinition[i].TYPE  of
				'double': aTYPE=1l
				'DOUBLE': aTYPE=1l 
				'float':  aTYPE=2l
				'FLOAT':  aTYPE=2l
				'long':   aTYPE=3l
				'LONG':   aTYPE=3l
				'short':  aTYPE=4l
				'SHORT':  aTYPE=4l
				'string': aTYPE=5l
				'STRING': aTYPE=5l
				'byte':  aTYPE=6l
				'BYTE':   aTYPE=6l
				ELSE:  Message, 'For ' + aDefinition[i].NAME + ' type not supported'
			end
		end
		aDESCRIPTION=''
		if( (where(tag_names(aDefinition[i]) EQ 'DESCRIPTION'))[0] NE -1)  then $
						aDESCRIPTION=aDefinition[i].DESCRIPTION
		aSYMBOL=''
		if( (where(tag_names(aDefinition[i]) EQ 'SYMBOL'))[0] NE -1)  then $
						aSYMBOL=aDefinition[i].SYMBOL
		aUNITS=''
		if( (where(tag_names(aDefinition[i]) EQ 'UNITS'))[0] NE -1)  then $
						aUNITS=aDefinition[i].UNITS
		aFORMAT_STRING=''
		if( (where(tag_names(aDefinition[i]) EQ 'FORMAT_STRING'))[0] NE -1)  then $
						aFORMAT_STRING=aDefinition[i].FORMAT_STRING
		aFIXED_VALUE=''
		if( (where(tag_names(aDefinition[i]) EQ 'FIXED_VALUE'))[0] NE -1)  then $
						aFIXED_VALUE=aDefinition[i].FIXED_VALUE
		if(call_external( $
			'libsddsIDL.so', $
			'setParameterDefinition',self.fd, $
			aNAME, $
			aSYMBOL, $
			aUNITS, $
			aDESCRIPTION, $
			aFORMAT_STRING, $
			aFIXED_VALUE, $
			aTYPE, $
			errMsg $
			) $
		  EQ  -1) then MESSAGE, 'ERROR setting parameter definition for ' + aDefinition[i].NAME $
					+ ' ' + string(errmsg)
	end
	return
end
;*****************************************************************************
pro sdds_table::setColumnDefinition, aDefinition
;+
; NAME:
;       SDDS_TABLE::SETCOLUMNDEFINITION
;
; PURPOSE:
;       Set the description of a column for SDDS table.
;
; CATEGORY:
;       SDDS interface.
;
; CALLING SEQUENCE:
;      setColumnDefinition, aDefinition
;
; INPUTS:
;	aDefinition structure or array of structures
;
; OUTPUTS:
;       An exception will be generated on error and ERR_STRING set.
;
;
; EXAMPLE:
;	aDefinition={columnDefinition,NAME:'',SYMBOL:'',UNITS:'',DESCRIPTION:'', $
;		     FORMAT_STRING:'',TYPE:'',FIELD_LENGTH:0l}
;	catch, error_status
;	if(error_status NE 0) then begin
;		print, 'ERROR_MESSAGE:',!ERR_STRING
;		return
;	endif
;       setColumnDefinition, aDefinition
;
; MODIFICATION HISTORY:
;       Written by:     Robert Daly April 27,1999
;-
	errMsg=bytarr(120)
	i=0l
	
	if( self.mode NE 'WRITE') then MESSAGE, ' File ' + self.file_name $
		+ ' not opened for write'
	if( self.free  NE 1) then MESSAGE, ' File ' + self.file_name $
		+ ' not initialized'
	if n_elements(aDefinition) EQ  0 then MESSAGE, 'Must define at least one parameter'
		
	for i=0, i<n_elements(aDefinition) do begin
		if( (where(tag_names(aDefinition[i]) EQ 'NAME'))[0] EQ  -1)  then $
			MESSAGE, 'NAME must be specified in definition'
		aNAME=aDefinition.NAME
		aTYPE=1l
		if( (where(tag_names(aDefinition[i]) EQ 'TYPE'))[0] NE -1) then begin
			case 	aDefinition[i].TYPE  of
				'double': aTYPE=1l
				'DOUBLE': aTYPE=1l 
				'float':  aTYPE=2l
				'FLOAT':  aTYPE=2l
				'long':   aTYPE=3l
				'LONG':   aTYPE=3l
				'short':  aTYPE=4l
				'SHORT':  aTYPE=4l
				'string': aTYPE=5l
				'STRING': aTYPE=5l
				'byte':  aTYPE=6l
				'BYTE':   aTYPE=6l
				DEFAULT:  Message, 'For ' + aDefinition[i].NAME + ' type not supported'
			end
		end
		aDESCRIPTION=''
		if( (where(tag_names(aDefinition[i]) EQ 'DESCRIPTION'))[0] NE -1)  then $
						aDESCRIPTION=aDefinition[i].DESCRIPTION
		aSYMBOL=''
		if( (where(tag_names(aDefinition[i]) EQ 'SYMBOL'))[0] NE -1)  then $
						aSYMBOL=aDefinition[i].SYMBOL
		aUNITS=''
		if( (where(tag_names(aDefinition[i]) EQ 'UNITS'))[0] NE -1)  then $
						aUNITS=aDefinition[i].UNITS
		aFORMAT_STRING=''
		if( (where(tag_names(aDefinition[i]) EQ 'FORMAT_STRING'))[0] NE -1)  then $
						aFORMAT_STRING=aDefinition[i].FORMAT_STRING
		aFIELD_LENGTH=0l
		if( (where(tag_names(aDefinition[i]) EQ 'FIELD_LENGTH'))[0] NE -1)  then $
						aFIELD_LENGTH=aDefinition[i].FIELD_LENGTH
		if(call_external( $
			'libsddsIDL.so', $
			'setColumnDefinition',self.fd, $
			aNAME, $
			aSYMBOL, $
			aUNITS, $
			aDESCRIPTION, $
			aFORMAT_STRING, $
			aFIELD_LENGTH, $
			aTYPE, $
			errMsg $
			) $
		  EQ  -1) then MESSAGE, 'ERROR setting column  definition for ' + aDefinition[i].NAME $
					+ ' ' + string(errmsg)
	end
	return
end
;*****************************************************************************
pro sdds_table::setArrayDefinition, aDefinition
;+
; NAME:
;       SDDS_TABLE::SETARRAYDEFINITION
;
; PURPOSE:
;       Set the description of a array for SDDS table.
;
; CATEGORY:
;       SDDS interface.
;
; CALLING SEQUENCE:
;      setArrayDefinition, aDefinition
;
; INPUTS:
;	aDefinition structure or array of structures
;
; OUTPUTS:
;       An exception will be generated on error and ERR_STRING set.
;
;
; EXAMPLE:
;	aDefinition={arrayDefinition,NAME:'',SYMBOL:'',UNITS:'',DESCRIPTION:'', $
;		     FORMAT_STRING:'',TYPE:'',FIELD_LENGTH:0l,DIMENSIONS:0l,GROUP_NAME:''}
;	catch, error_status
;	if(error_status NE 0) then begin
;		print, 'ERROR_MESSAGE:',!ERR_STRING
;		return
;	endif
;       setArrayDefinition, aDefinition
;
; MODIFICATION HISTORY:
;       Written by:     Robert Daly April 27,1999
;-
	errMsg=bytarr(120)
	i=0l
	
	if( self.mode NE 'WRITE') then MESSAGE, ' File ' + self.file_name $
		+ ' not opened for write'
	if( self.free  NE 1) then MESSAGE, ' File ' + self.file_name $
		+ ' not initialized'
	if n_elements(aDefinition) EQ  0 then MESSAGE, 'Must define at least one parameter'
		
	for i=0, i<n_elements(aDefinition) do begin
		if( (where(tag_names(aDefinition[i]) EQ 'NAME'))[0] EQ  -1)  then $
			MESSAGE, 'NAME must be specified in definition'
		aNAME=aDefinition.NAME
		aTYPE=1l
		if( (where(tag_names(aDefinition[i]) EQ 'TYPE'))[0] NE -1) then begin
			case 	aDefinition[i].TYPE  of
				'double': aTYPE=1l
				'DOUBLE': aTYPE=1l 
				'float':  aTYPE=2l
				'FLOAT':  aTYPE=2l
				'long':   aTYPE=3l
				'LONG':   aTYPE=3l
				'short':  aTYPE=4l
				'SHORT':  aTYPE=4l
				'string': aTYPE=5l
				'STRING': aTYPE=5l
				'byte':  aTYPE=6l
				'BYTE':   aTYPE=6l
				else:  Message, 'For ' + aDefinition[i].NAME + ' type not supported'
			end
		end
		aDESCRIPTION=''
		if( (where(tag_names(aDefinition[i]) EQ 'DESCRIPTION'))[0] NE -1)  then $
						aDESCRIPTION=aDefinition[i].DESCRIPTION
		aSYMBOL=''
		if( (where(tag_names(aDefinition[i]) EQ 'SYMBOL'))[0] NE -1)  then $
						aSYMBOL=aDefinition[i].SYMBOL
		aUNITS=''
		if( (where(tag_names(aDefinition[i]) EQ 'UNITS'))[0] NE -1)  then $
						aUNITS=aDefinition[i].UNITS
		aFORMAT_STRING=''
		if( (where(tag_names(aDefinition[i]) EQ 'FORMAT_STRING'))[0] NE -1)  then $
						aFORMAT_STRING=aDefinition[i].FORMAT_STRING
		aFIELD_LENGTH=0l
		if( (where(tag_names(aDefinition[i]) EQ 'FIELD_LENGTH'))[0] NE -1)  then $
						aFIELD_LENGTH=aDefinition[i].FIELD_LENGTH
		aDIMENSIONS=2l
		if( (where(tag_names(aDefinition[i]) EQ 'DIMENSIONS'))[0] NE -1)  then $
						aDIMENSIONS=aDefinition[i].DIMENSIONS
		aGROUP_NAME=''
		if( (where(tag_names(aDefinition[i]) EQ 'GROUP_NAME'))[0] NE -1)  then $
						aGROUP_NAME=aDefinition[i].GROUP_NAME
		if(call_external( $
			'libsddsIDL.so', $
			'setArrayDefinition',self.fd, $
			aNAME, $
			aSYMBOL, $
			aUNITS, $
			aDESCRIPTION, $
			aFORMAT_STRING, $
			aFIELD_LENGTH, $
			aDIMENSIONS, $
			aGROUP_NAME, $
			aTYPE, $
			errMsg $
			) $
		  EQ  -1) then MESSAGE, 'ERROR setting array  definition for ' + aDefinition[i].NAME $
					+ ' ' + string(errmsg)
	end
	return
end

;*****************************************************************************
pro sdds_table::setParameter, name, aValue
;+
; NAME:
;       SDDS_TABLE::SETPARAMETER
;
; PURPOSE:
;       Set the value  of a parameter for SDDS table.
;
; CATEGORY:
;       SDDS interface.
;
; CALLING SEQUENCE:
;      setParameter, name, aValue
;
; INPUTS:
;	aValue  value to set patameter
;
; OUTPUTS:
;       An exception will be generated on error and ERR_STRING set.
;
;
; EXAMPLE:
;	aValue=230
;	catch, error_status
;	if(error_status NE 0) then begin
;		print, 'ERROR_MESSAGE:',!ERR_STRING
;		return
;	endif
;       setParameter, name, aValue)
;
; MODIFICATION HISTORY:
;       Written by:     Robert Daly April 27,1999
;-
	errMsg=bytarr(120)
	i=0l

	if( self.mode NE 'WRITE') then MESSAGE, ' File ' + self.file_name $
		+ ' not opened for write'
	if( self.free  NE 1) then MESSAGE, ' File ' + self.file_name $
		+ ' not initialized'
	result=self->sdds_table::getParameterDefinition(name)
	if(self.pageStarted EQ 0) then MESSAGE, 'ERROR setParameter for ' + name + ' table not started'
		
	
	on_ioerror, trapio
	case  result.TYPE  of
	'DOUBLE': begin
		retVal=DOUBLE(aValue)
		result=call_external('libsddsIDL.so', $
					'setParameterValueD',self.fd,name,retVal,errMsg)
	   end

	'FLOAT': begin
		retVal=FLOAT(aValue)
		result=call_external('libsddsIDL.so', $
					'setParameterValueF',self.fd,name,retVal,errMsg)
	   end
	'LONG': begin
		retVal=LONG(aValue)
		result=call_external('libsddsIDL.so', $
					'setParameterValueL',self.fd,name,retVal,errMsg)
	   end
	'SHORT': begin
		retVal=FIX(aValue)
		result=call_external('libsddsIDL.so', $
					'setParameterValueS',self.fd,name,retVal,errMsg)
	   end
	'STRING': begin
		retVal=STRING(aValue)
		result=call_external('libsddsIDL.so', $
					'setParameterValueString',self.fd,name,retVal,errMsg)
	   end

	'BYTE': begin
		retVal=BYTE(aValue)
		result=call_external('libsddsIDL.so', $
					'setParameterValueB',self.fd,name,retVal,errMsg)
	   end
	ELSE:  Message , 'Name ' + name + ' ' + 'type not supported'
	end 

	if(result  EQ  0) then Message , 'Name ' + name + ' ' + string(errmsg)

	GOTO, done
	trapio: MESSAGE, 'Name ' + name + ' ' + !ERR_STRING, /NONAME
	done:
	return


end

;*****************************************************************************
pro sdds_table::setColumn, name, aValue
;+
; NAME:
;       SDDS_TABLE::SETCOLUMN
;
; PURPOSE:
;       Set the values  of a column for SDDS table.
;
; CATEGORY:
;       SDDS interface.
;
; CALLING SEQUENCE:
;      setColumn, name, aValue
;
; INPUTS:
;	aValue  value(s) to set column
;
; OUTPUTS:
;       An exception will be generated on error and ERR_STRING set.
;
;
; EXAMPLE:
;	aValue=230
;	catch, error_status
;	if(error_status NE 0) then begin
;		print, 'ERROR_MESSAGE:',!ERR_STRING
;		return
;	endif
;       setColumn, name, aValue
;
; MODIFICATION HISTORY:
;       Written by:     Robert Daly April 27,1999
;-
	errMsg=bytarr(120)
	i=0l

	if( self.mode NE 'WRITE') then MESSAGE, ' File ' + self.file_name $
		+ ' not opened for write'
	if( self.free  NE 1) then MESSAGE, ' File ' + self.file_name $
		+ ' not initialized'
	elements=long(n_elements(aValue))
	if(elements EQ 0) then Message ,'setColumn must supply value(s)'
	if(self.pageStarted EQ 0) then MESSAGE, 'ERROR setColumn for ' + name + ' table not started'
	definition=self->sdds_table::getColumnDefinition(name)
	on_ioerror, trapio
	case  definition.TYPE  of
	'DOUBLE': begin
		retVal=DOUBLE(aValue)
		result=call_external('libsddsIDL.so', $
					'setColumnValueD',self.fd,name,retVal,elements,errMsg)
	   end

	'FLOAT': begin
		retVal=FLOAT(aValue)
		result=call_external('libsddsIDL.so', $
					'setColumnValueF',self.fd,name,retVal,elements,errMsg)
	   end
	'LONG': begin
		retVal=LONG(aValue)
		result=call_external('libsddsIDL.so', $
					'setColumnValueL',self.fd,name,retVal,elements,errMsg)
	   end
	'SHORT': begin
		retVal=FIX(aValue)
		result=call_external('libsddsIDL.so', $
					'setColumnValueS',self.fd,name,retVal,elements,errMsg)
	   end
	'STRING': begin
		retVal=STRING(aValue)
		result=call_external('libsddsIDL.so', $
					'setColumnValueString',self.fd,name,retVal,elements,errMsg)
	   end

	'BYTE': begin
		retVal=BYTE(aValue)
		result=call_external('libsddsIDL.so', $
					'setColumnValueB',self.fd,name,retVal,elements,errMsg)
	   end
	ELSE:  Message , 'Name ' + name + ' ' + 'type not supported'
	end 

	if(result  EQ  0) then Message , 'Name ' + name + ' ' + string(errmsg)

	GOTO, done
	trapio: MESSAGE, 'Name ' + name + ' ' + !ERR_STRING, /NONAME
	done:
	return


end
;*****************************************************************************
pro sdds_table::setArray, name, aArray
;+
; NAME:
;       SDDS_TABLE::SETARRAY
;
; PURPOSE:
;       Set the values  of an array for SDDS table.
;
; CATEGORY:
;       SDDS interface.
;
; CALLING SEQUENCE:
;      setArray, name, aArray
;
; INPUTS:
;	aArray  
;
; OUTPUTS:
;       An exception will be generated on error and ERR_STRING set.
;
;
; EXAMPLE:
;	aArray=MAKE_ARRAY(512,512,/DOUBLE,VALUE=1.1D)
;	catch, error_status
;	if(error_status NE 0) then begin
;		print, 'ERROR_MESSAGE:',!ERR_STRING
;		return
;	endif
;       setArray, name, aArray
;
; MODIFICATION HISTORY:
;       Written by:     Robert Daly April 27,1999
;-
	errMsg=bytarr(120)
	i=0l

	if( self.mode NE 'WRITE') then MESSAGE, ' File ' + self.file_name $
		+ ' not opened for write'
	if( self.free  NE 1) then MESSAGE, ' File ' + self.file_name $
		+ ' not initialized'
	if(n_params() NE 2) then MESSAGE, 'setArray must supply both name and data'
	definition=self->sdds_table::getArrayDefinition(name)
	aSize=size(aArray)
	if(aSize(0) NE definition.DIMENSIONS) then MESSAGE, 'setArray wrong dimensions expect:  ' $
		 + string(definition.DIMENSIONS) + ' supplied:  ' + string(aSize(0))
	dimArr=lonarr(aSize(0))
	for i=0,  aSize(0)-1 do dimArr[i]=aSize[i+1]
	
	on_ioerror, trapio
	case  definition.TYPE  of
	'DOUBLE': begin
		retVal=DOUBLE(aArray)
		result=call_external('libsddsIDL.so', $
					'setArrayValueD',self.fd,name,retVal,dimArr,errMsg)
	   end

	'FLOAT': begin
		retVal=FLOAT(aArray)
		result=call_external('libsddsIDL.so', $
					'setArrayValueF',self.fd,name,retVal,dimArr,errMsg)
	   end
	'LONG': begin
		retVal=LONG(aArray)
		result=call_external('libsddsIDL.so', $
					'setArrayValueL',self.fd,name,retVal,dimArr,errMsg)
	   end
	'SHORT': begin
		retVal=FIX(aArray)
		result=call_external('libsddsIDL.so', $
					'setArrayValueS',self.fd,name,retVal,dimArr,errMsg)
	   end
	'STRING': begin
		elements=long(n_elements(aArray))
		retVal=STRING(aArray)
		result=call_external('libsddsIDL.so', $
					'setArrayValueString',self.fd,name,retVal,dimArr,elements,errMsg)
	   end

	'BYTE': begin
		retVal=BYTE(aArray)
		result=call_external('libsddsIDL.so', $
					'setArrayValueB',self.fd,name,retVal,dimArr,errMsg)
	   end
	ELSE:  Message , 'Name ' + name + ' ' + 'type not supported'
	end 

	if(result  EQ  0) then Message , 'Name ' + name + ' ' + string(errmsg)

	GOTO, done
	trapio: MESSAGE, 'Name ' + name + ' ' + !ERR_STRING, /NONAME
	done:
	return


end

;*****************************************************************************
pro sdds_table::setRow, name, aValue, arow
;+
; NAME:
;       SDDS_TABLE::SETCOLUMN
;
; PURPOSE:
;       Set the value  of a row element for SDDS table.
;
; CATEGORY:
;       SDDS interface.
;
; CALLING SEQUENCE:
;      setRow, name, aValue, arow
;
; INPUTS:
;	aValue  value(s) to set roe
;
; OUTPUTS:
;       An exception will be generated on error and ERR_STRING set.
;
;
; EXAMPLE:
;	aValue=230
;	row = 1l
;	catch, error_status
;	if(error_status NE 0) then begin
;		print, 'ERROR_MESSAGE:',!ERR_STRING
;		return
;	endif
;       setColumn, name, aValue, row
;
; MODIFICATION HISTORY:
;       Written by:     Robert Daly April 27,1999
;-
	errMsg=bytarr(120)
	i=0l

	if( self.mode NE 'WRITE') then MESSAGE, ' File ' + self.file_name $
		+ ' not opened for write'
	if( self.free  NE 1) then MESSAGE, ' File ' + self.file_name $
		+ ' not initialized'
	elements=long(n_elements(aValue))
	if(elements EQ 0) then Message ,'setRow must supply value'
	elements=long(n_elements(arow))
	if(elements EQ 0) then Message ,'setRow must supply row number'
	if(self.pageStarted EQ 0) then MESSAGE, 'ERROR setRow for ' + name + ' table not started'
	definition=self->sdds_table::getColumnDefinition(name)
		
	on_ioerror, trapio
	row=long(arow)
	case  definition.TYPE  of
	'DOUBLE': begin
		retVal=DOUBLE(aValue)
		result=call_external('libsddsIDL.so', $
					'setRowValueD',self.fd,name,retVal,row,errMsg)
	   end

	'FLOAT': begin
		retVal=FLOAT(aValue)
		result=call_external('libsddsIDL.so', $
					'setRowValueF',self.fd,name,retVal,row,errMsg)
	   end
	'LONG': begin
		retVal=LONG(aValue)
		result=call_external('libsddsIDL.so', $
					'setRowValueL',self.fd,name,retVal,row,errMsg)
	   end
	'SHORT': begin
		retVal=FIX(aValue)
		result=call_external('libsddsIDL.so', $
					'setRowValueS',self.fd,name,retVal,row,errMsg)
	   end
	'STRING': begin
		retVal=STRING(aValue)
		result=call_external('libsddsIDL.so', $
					'setRowValueString',self.fd,name,retVal,row,errMsg)
	   end

	'BYTE': begin
		retVal=BYTE(aValue)
		result=call_external('libsddsIDL.so', $
					'setRowValueB',self.fd,name,retVal,row,errMsg)
	   end
	ELSE:  Message , 'Name ' + name + ' ' + 'type not supported'
	end 

	if(result  EQ  0) then Message , 'Name ' + name + ' ' + string(errmsg)

	GOTO, done
	trapio: MESSAGE, 'Name ' + name + ' ' + !ERR_STRING, /NONAME
	done:
	return


end


;*****************************************************************************
function sdds_table::getArrayDefinition, name
;+
; NAME:
;       SDDS_TABLE::GETARRAYDEFINITION
;
; PURPOSE:
;       Get the description of a array from SDDS table.
;
; CATEGORY:
;       SDDS interface.
;
; CALLING SEQUENCE:
;       Result = getArrayDefinition(name)
;
; INPUTS:
;	aDefinition structure to be filled in by function
;
; OUTPUTS:
;       This function returns  a Definition of am  SDDS array 
;
;
; EXAMPLE:
;	aDefinition={arrayDefinition,NAME:'',SYMBOL:'',UNITS:'',DESCRIPTION:'', $
;		     FORMAT_STRING:'',GROUP_NAME: '',TYPE:'',FIELD_LENGTH:0l,DIMENSIONS:0l}
;	catch, error_status
;	if(error_status NE 0) then begin
;		print, 'ERROR_MESSAGE:',!ERR_STRING
;		return
;	endif
;       Result= getArrayDefinition('anArrayName')
;
; MODIFICATION HISTORY:
;       Written by:     Robert Daly April 27,1999
;-
	status=0
	errMsg=bytarr(120)
	type=0l
	field_length = 0l
	dimensions=0l
	arrayDefinition=call_external('libsddsIDL.so', $
					'getArrayDefinition',self.fd,name,type,field_length, $
					dimensions,status,errMsg,/S_VALUE)

	if(status EQ 0) then MESSAGE, 'Name: ' + name + string(errMsg)
	aDefinition={NAME:'',SYMBOL:'',UNITS:'',DESCRIPTION:'', $
		     FORMAT_STRING:'',GROUP_NAME: '',TYPE:'',FIELD_LENGTH:0l,DIMENSIONS:0l}
	arrayArray=STR_SEP(arrayDefinition,"*")
	aDefinition.NAME=arrayArray(0)
	aDefinition.SYMBOL=arrayArray(1)
	aDefinition.UNITS=arrayArray(2)
	aDefinition.DESCRIPTION=arrayArray(3)
	aDefinition.FORMAT_STRING=arrayArray(4)
	aDefinition.GROUP_NAME=arrayArray(4)
	case type  of
		1l:aDefinition.TYPE='DOUBLE' 
		2l:aDefinition.TYPE='FLOAT'
		3l:aDefinition.TYPE='LONG'
		4l:aDefinition.TYPE='SHORT'
		5l:aDefinition.TYPE='STRING'
		6l:aDefinition.TYPE='BYTE' 
	end
	aDefinition.FIELD_LENGTH=field_length
	aDefinition.DIMENSIONS=dimensions

	return, aDefinition	
end
;*****************************************************************************
function sdds_table::getParameterDefinition, name
;+
; NAME:
;       SDDS_TABLE::GETPARAMETERDEFINITION
;
; PURPOSE:
;       Get the description of a parameter from SDDS table.
;
; CATEGORY:
;       SDDS interface.
;
; CALLING SEQUENCE:
;       Result = getParameterDefinition(name,aDefinition)
;
; INPUTS:
;	aDefinition structure to be filled in by function
;
; OUTPUTS:
;       This function returns in aDefinition a SDDS parameter definition
;	and the result(-1,0) in Result.
;
;
; EXAMPLE:
;	aDefinition={parameterDefinition,NAME:'',SYMBOL:'',UNITS:'',DESCRIPTION:'', $
;		     FORMAT_STRING:'',FIXED_VALUE:'',TYPE:0l}
;       aDefinition= getParameterDefinition('aParameterName')
;
; MODIFICATION HISTORY:
;       Written by:     Robert Daly April 27,1999
;-
	status=0
	errMsg=bytarr(120)
	type=0l
	i=0
	
	parameterDefinition=call_external('libsddsIDL.so', $
					'getParameterDefinition',self.fd,name,type,status,errMsg,/S_VALUE)

	if(status EQ 0) then MESSAGE, 'For ' + name +' ' + string(errMsg)
	parameterArray=STR_SEP(parameterDefinition,"*")
	aDefinition={NAME:'',SYMBOL:'',UNITS:'',DESCRIPTION:'', $
		     FORMAT_STRING:'',FIXED_VALUE:'',TYPE:''}
	aDefinition.NAME=parameterArray(0)
	aDefinition.SYMBOL=parameterArray(1)
	aDefinition.UNITS=parameterArray(2)
	aDefinition.DESCRIPTION=parameterArray(3)
	aDefinition.FORMAT_STRING=parameterArray(4)
	aDefinition.FIXED_VALUE=parameterArray(5)
	case type  of
		1l:aDefinition.TYPE='DOUBLE' 
		2l:aDefinition.TYPE='FLOAT'
		3l:aDefinition.TYPE='LONG'
		4l:aDefinition.TYPE='SHORT'
		5l:aDefinition.TYPE='STRING'
		6l:aDefinition.TYPE='BYTE' 
	end

	return, aDefinition	
end


;*****************************************************************************
function sdds_table::getArrayNames
;+
; NAME:
;       SDDS_TABLE::GETARRAYNAMES
;
; PURPOSE:
;       Get the names of a class of variable from a SDDS table.
;
; CATEGORY:
;       SDDS interface.
;
; CALLING SEQUENCE:
;       Result = getArrayNames()
;
; INPUTS:
;
; OUTPUTS:
;       This function returns an array of names.
;
;
; EXAMPLE:
;       Result= getArrayNames()
;
; MODIFICATION HISTORY:
;       Written by:     Robert Daly April 27,1999
;-
	if(self.mode NE "READ") then MESSAGE, 'Array names not available in /NEW mode'
	if(self.free NE 1     ) then MESSAGE, 'Initialization error'
	if self.arrays EQ 0  then return, ' '
	return, *(self.arrayInfo)	
end
;*****************************************************************************
function sdds_table::getArrayNumber
;+
; NAME:
;       SDDS_TABLE::GETARRAYNumber
;
; PURPOSE:
;       Get the number of arrays defined.
;
; CATEGORY:
;       SDDS interface.
;
; CALLING SEQUENCE:
;       Result = getArrayNumber()
;
; INPUTS:
;
; OUTPUTS:
;       This function returns the number of arrays defined.
;
;
; EXAMPLE:
;       Result= getArrayNumber()
;
; MODIFICATION HISTORY:
;       Written by:     Robert Daly April 27,1999
;-
	if(self.mode NE "READ") then MESSAGE, 'Array names not available in /NEW mode'
	if(self.free NE 1     ) then MESSAGE, 'Initialization error'
	return, self.arrays	
end
;*****************************************************************************
function sdds_table::getColumnNumber
;+
; NAME:
;       SDDS_TABLE::GETCOLUMNNUMBER
;
; PURPOSE:
;       Get the number of columns defined.
;
; CATEGORY:
;       SDDS interface.
;
; CALLING SEQUENCE:
;       Result = getColumnNumber()
;
; INPUTS:
;
; OUTPUTS:
;       This function returns the number of columns defined.
;
;
; EXAMPLE:
;       Result= getColumnNumber()
;
; MODIFICATION HISTORY:
;       Written by:     Robert Daly April 27,1999
;-
	if(self.mode NE "READ") then MESSAGE, 'Array names not available in /NEW mode'
	if(self.free NE 1     ) then MESSAGE, 'Initialization error'
	return, self.columns	
end
;*****************************************************************************
function sdds_table::getParameterNumber
;+
; NAME:
;       SDDS_TABLE::GETPARAMETERUMBER
;
; PURPOSE:
;       Get the number of parameters defined.
;
; CATEGORY:
;       SDDS interface.
;
; CALLING SEQUENCE:
;       Result = getParameterNumber()
;
; INPUTS:
;
; OUTPUTS:
;       This function returns the number of parameters defined.
;
;
; EXAMPLE:
;       Result= getParameterNumber()
;
; MODIFICATION HISTORY:
;       Written by:     Robert Daly April 27,1999
;-
	if(self.mode NE "READ") then MESSAGE, 'Array names not available in /NEW mode'
	if(self.free NE 1     ) then MESSAGE, 'Initialization error'
	return, self.parameters	
end
;*****************************************************************************
function sdds_table::readTable, page
;+
; NAME:
;       SDDS_TABLE::READPAGE
;
; PURPOSE:
;       Read a table from an SDDS file.
;
; CATEGORY:
;       SDDS interface.
;
; CALLING SEQUENCE:
;       Result = readTable(page)
;
; INPUTS:
;	page to read, if page not specified then read next page
;
; OUTPUTS:
;       This function returns the current page after read, or -1 for End 0f File.
;
;
; EXAMPLE:
;       Result= readTable()
;
; MODIFICATION HISTORY:
;       Written by:     Robert Daly April 27,1999
;-
	errMsg=bytarr(120)
	requestedPage=0
	status=0l
	if(self.mode NE "READ") then MESSAGE, 'Read page not available in /NEW mode'
	if(self.free NE 1     ) then MESSAGE, 'Initialization error'
	if(n_params() EQ 0) then requestedPage=-1 else $
	if(page EQ 0) then requestedPage=1 else $
	 requestedPage=page
	currentPage=call_external('libsddsIDL.so', $
					'setPage',self.fd,requestedPage,status,errMsg)
	
	if(status EQ 0)	then MESSAGE, 'setPage(): ' + string(errmsg)
	return, currentPage	
end
;*****************************************************************************
function sdds_table::getArray, name
;+
; NAME:
;       SDDS_TABLE::GETARRAY
;
; PURPOSE:
;       Get the value of a array from SDDS table.
;
; CATEGORY:
;       SDDS interface.
;
; CALLING SEQUENCE:
;       Result = getArray(name)
;
; INPUTS:
;	array name 
;
; OUTPUTS:
;       This function returns in Result the  data array.
;
;
; EXAMPLE:
;	catch, error_status
;	if(error_status NE 0) then begin
;		print, 'ERROR_MESSAGE:',!ERR_STRING
;		return
;	endif
;       Result= getArray('aArrayName')
;
; MODIFICATION HISTORY:
;       Written by:     Robert Daly April 27,1999
;-
	
	status=0
	errMsg=bytarr(120)
	requestedPage=-1
	arrayType=0l
	arrayDimensions=0l
	arrayElements=0l
	arrayDim=lonarr(4)
	i=0l
	j=0l
	k=0l
	l=0l

	if(self.mode NE "READ") then MESSAGE, 'Read page not available in /NEW mode'
	if(self.free NE 1     ) then MESSAGE, 'Initialization error'
	if(self.arrays EQ 0) then MESSAGE, 'No arrays defined in ' + self.file_name
	index=0

	index=where(*(self.arrayInfo) EQ name)
	if(index[0] eq -1) then MESSAGE, 'array name ' + name + '  not found'
	definition=self->sdds_table::getArrayDefinition(name)
	if( definition.DIMENSIONS  GT 4 ) then MESSAGE, 'Supports up to 4 dimensions'
	elements=call_external('libsddsIDL.so', $
				'getArrayData',self.fd,name,arrayType,arrayDimensions,arrayDim,status,errMsg)

	if status EQ 0 then MESSAGE, 'getArrayValue:elements ' + name + ' ' + string(errmsg)
	if( arrayDimensions  GT 4l ) then MESSAGE, 'Supports up to 4 dimensions'
	case  definition.TYPE of
	'DOUBLE': begin
		case arrayDimensions of
		1l: retVal=dblarr(arrayDim[0])
		2l: retVal=dblarr(arrayDim[0],arrayDim[1])
		3l: retVal=dblarr(arrayDim[0],arrayDim[1],arrayDim[2])
		4l: retVal=dblarr(arrayDim[0],arrayDim[1],arrayDim[2],arrayDim[3])
		end
		result=call_external('libsddsIDL.so', $
					'getArrayValueD',self.fd,retVal,status,errMsg)
	   end

	'FLOAT': begin
		case arrayDimensions of
		1l: retVal=fltarr(arrayDim[0])
		2l: retVal=fltarr(arrayDim[0],arrayDim[1])
		3l: retVal=fltarr(arrayDim[0],arrayDim[1],arrayDim[2])
		4l: retVal=fltarr(arrayDim[0],arrayDim[1],arrayDim[2],arrayDim[3])
		end
		result=call_external('libsddsIDL.so', $
					'getArrayValueF',self.fd,retVal,status,errMsg)
	   end
	'LONG': begin
		case arrayDimensions of
		1l: retVal=lonarr(arrayDim[0])
		2l: retVal=lonarr(arrayDim[0],arrayDim[1])
		3l: retVal=lonarr(arrayDim[0],arrayDim[1],arrayDim[2])
		4l: retVal=lonarr(arrayDim[0],arrayDim[1],arrayDim[2],arrayDim[3])
		end
		result=call_external('libsddsIDL.so', $
					'getArrayValueL',self.fd,retVal,status,errMsg)
	   end
	'SHORT': begin
		case arrayDimensions of
		1l: retVal=intarr(arrayDim[0])
		2l: retVal=intarr(arrayDim[0],arrayDim[1])
		3l: retVal=intarr(arrayDim[0],arrayDim[1],arrayDim[2])
		4l: retVal=intarr(arrayDim[0],arrayDim[1],arrayDim[2],arrayDim[3])
		end
		result=call_external('libsddsIDL.so', $
					'getArrayValueS',self.fd,retVal,status,errMsg)
	   end
	'STRING': begin
		result=call_external('libsddsIDL.so', $
					'getArrayValueString',self.fd,status,errMsg,/S_VALUE)
		retArr=STR_SEP(result,'*')
		case arrayDimensions of
		1l: retVal=retArr
		2l: begin
			retVal=strarr(arrayDim[0],arrayDim[1])
			for i=0, arrayDim[1]-1 do begin
				for j=0,arrayDim[0]-1 do begin
					retVal[j,i]=retArr[j+i*arrayDim[0]]
				end
			end
		  end
		3l: begin
 			retVal=strarr(arrayDim[0],arrayDim[1],arrayDim[2])
			dim10=arrayDim[1]*arrayDim[0]
			for i=0,arrayDim[2]-1 do begin
				for j=0, arrayDim[1]-1 do begin
					for k=0, arrayDim[0]-1 do begin
						retVal[k,j,i]=retArr[k+j*arrayDim[0] +i*dim10]
					end
				end
			end
		   end

		4l: begin
			retVal=strarr(arrayDim[0],arrayDim[1],arrayDim[2],arrayDim[3])
			dim10=arrayDim[1]*arrayDim[0]
			dim210=arrayDim[2]*arrayDim[1]*arrayDim[0]
			for i=0, arrayDim[3]-1 do begin
				for j=0,arrayDim[2]-1 do begin
					for k=0, arrayDim[1]-1 do begin
						for l=0, arrayDim[0]-1 do begin
							retVal[l,k,j,i]=retArr[l+k*arrayDim[0] +j*dim10 +i*dim210]
						end
					end
				end
			end
		   end

		end
		end
	'BYTE': begin
		case arrayDimensions of
		1l: retVal=bytarr(arrayDim[0])
		2l: retVal=bytarr(arrayDim[0],arrayDim[1])
		3l: retVal=bytarr(arrayDim[0],arrayDim[1],arrayDim[2])
		4l: retVal=bytarr(arrayDim[0],arrayDim[1],arrayDim[2],arrayDim[3])
		end
		result=call_external('libsddsIDL.so', $
					'getArrayValueB',self.fd,retVal,status,errMsg)
	   end
	5: MESSAGE, 'String arrays not supported'
	ELSE: MESSAGE, 'getArrayValue TYPE not valid'
	end 

	if(status EQ  0) then MESSAGE, 'getArrayValue:error  ' + name + string(errMsg)
	

	return, retVal	
end
;*****************************************************************************
function sdds_table::getColumnNames
;+
; NAME:
;       SDDS_TABLE::GETCOLUMNNAMES
;
; PURPOSE:
;       Get the names of columns from a SDDS table.
;
; CATEGORY:
;       SDDS interface.
;
; CALLING SEQUENCE:
;       Result = getColumnNames()
;
; INPUTS:
;
; OUTPUTS:
;       This function returns an array of column names as strings.
;
;
; EXAMPLE:
;       Result= getColumnNames()
;
; MODIFICATION HISTORY:
;       Written by:     Robert Daly April 27,1999
;-
	if(self.mode NE "READ") then MESSAGE, 'Read page not available in /NEW mode'
	if(self.free NE 1     ) then MESSAGE, 'Initialization error'
	if self.columns EQ 0 then return, ''
	return, *(self.columnInfo)	
end
;*****************************************************************************
function sdds_table::getParameterNames
;+
; NAME:
;       SDDS_TABLE::GETPARAMETERNAMES
;
; PURPOSE:
;       Get the names of parameters from a SDDS table.
;
; CATEGORY:
;       SDDS interface.
;
; CALLING SEQUENCE:
;       Result = getParameterNames()
;
; INPUTS:
;
; OUTPUTS:
;       This function returns an array of column names.
;
;
; EXAMPLE:
;       Result= getParameterNames()
;
; MODIFICATION HISTORY:
;       Written by:     Robert Daly April 27,1999
;-
	if(self.mode NE "READ") then MESSAGE, 'Read page not available in /NEW mode'
	if(self.free NE 1     ) then MESSAGE, 'Initialization error'
	if self.parameters EQ 0 then return, ''
	return, *(self.parameterInfo)
end
;*****************************************************************************
function sdds_table::getParameter, name
;+
; NAME:
;       SDDS_TABLE::GETPARAMETER
;
; PURPOSE:
;       Get the value of a parameter from SDDS table.
;
; CATEGORY:
;       SDDS interface.
;
; CALLING SEQUENCE:
;       Result = getParameter(name)
;
; INPUTS:
;	parameter name
;
; OUTPUTS:
;       This function returns a parameter value.
;
;
; EXAMPLE:
;       Result= getParameter('aParameterName')
;
; MODIFICATION HISTORY:
;       Written by:     Robert Daly April 27,1999
;-
	status=0
	errMsg=bytarr(120)
	if(self.mode NE "READ") then MESSAGE, 'Read page not available in /NEW mode'
	if(self.free NE 1     ) then MESSAGE, 'Initialization error'
	index=0
	index=where(*(self.parameterInfo) EQ name)
	if(index[0] EQ -1) then MESSAGE, 'getParametervalue: ' + name + ' not defined'
	definition=self->sdds_table::getParameterDefinition(name)
	case  definition.TYPE of
	'DOUBLE': begin
		retVal=0D
		result=call_external('libsddsIDL.so', $
					'getParameterValueD',self.fd,name,retVal,status,errMsg)
	   end

	'FLOAT': begin
		retVal=0E
		result=call_external('libsddsIDL.so', $
					'getParameterValueF',self.fd,name,retVal,status,errMsg)
	   end
	'LONG': begin
		retVal=0L
		result=call_external('libsddsIDL.so', $
					'getParameterValueL',self.fd,name,retVal,status,errMsg)
	   end
	'SHORT': begin
		retVal=0
		result=call_external('libsddsIDL.so', $
					'getParameterValueS',self.fd,name,retVal,status,errMsg)
	   end
	'STRING': begin
		retVal=call_external('libsddsIDL.so', $
					'getParameterValueString',self.fd,name,status,errMsg,/S_VALUE)
	   end
	'BYTE': begin
		retVal=0B
		result=call_external('libsddsIDL.so', $
					'getParameterValueB',self.fd,name,retVal,status,errMsg)
	   end
	ELSE: MESSAGE, 'getParameterValue: type ' + definition.TYPE + ' not supported'
	end 

	if(status EQ  0) then MESSAGE, 'getParameterValue for  ' + name + '  ' + string(errMsg)

	return, retVal	
end
;*****************************************************************************
function sdds_table::getColumn, name
;+
; NAME:
;       SDDS_TABLE::GETCOLUMN
;
; PURPOSE:
;       Get the value of a column from SDDS table.
;
; CATEGORY:
;       SDDS interface.
;
; CALLING SEQUENCE:
;       Result = getColumn(name)
;
; INPUTS:
;	column name a
;
; OUTPUTS:
;       This function returns in Result the column data vector.
;
;
; EXAMPLE
;	catch, error_status
;	if(error_status NE 0) then begin
;		print, 'ERROR_MESSAGE:',!ERR_STRING
;		return
;	endif
;       Result= getColumn('aColumnName')
;
; MODIFICATION HISTORY:
;       Written by:     Robert Daly April 27,1999
;-
	status=0
	errMsg=bytarr(120)
	if(self.mode NE "READ") then MESSAGE, 'Read page not available in /NEW mode'
	if(self.free NE 1     ) then MESSAGE, 'Initialization error'
	index=0
	while (index LT self.columns ) and ( ( *(self.columnInfo))[index] NE  name) do index=index+1
	
	if(index[0] eq self.columns) then MESSAGE, 'getColumnValue column ' +  name  + ' not found'
	self.rows=call_external('libsddsIDL.so', $
				'getColumnRows',self.fd,status,errMsg)
	if self.rows EQ -1 then MESSAGE, 'getColumnValue: ' + string(errmsg)
	description=self->sdds_table::getColumnDefinition(name)
	case  description.TYPE of
	'DOUBLE': begin
		retVal=dblarr(self.rows)
		result=call_external('libsddsIDL.so', $
					'getColumnValueD',self.fd,name,retVal,self.rows,status,errMsg)
	   end

	'FLOAT': begin
		retVal=fltarr(self.rows)
		result=call_external('libsddsIDL.so', $
					'getColumnValueF',self.fd,name,retVal,self.rows,status,errMsg)
	   end
	'LONG': begin
		retVal=lonarr(self.rows)
		result=call_external('libsddsIDL.so', $
					'getColumnValueL',self.fd,name,retVal,self.rows,status,errMsg)
	   end
	'SHORT': begin
		retVal=intarr(self.rows)
		result=call_external('libsddsIDL.so', $
					'getColumnValueS',self.fd,name,retVal,self.rows,status,errMsg)
	   end
	'STRING': begin
		result=call_external('libsddsIDL.so', $
					'getColumnValueString',self.fd,name,self.rows,status,errMsg,/S_VALUE)
		retVal=STR_SEP(result,"*")
	   end
	'BYTE': begin
		retVal=bytarr(self.rows)
		result=call_external('libsddsIDL.so', $
					'getColumnValueB',self.fd,name,retVal,self.rows,status,errMsg)
	   end
	ELSE: MESSAGE, 'getColumnValue TYPE not valid'
	end 

	if(status EQ  0) then MESSAGE, 'getColumnValue:' + name + ' ' +  string(errMsg)

	return, retVal	
end

;*****************************************************************************
function sdds_table::getColumnDefinition, name
;+
; NAME:
;       SDDS_TABLE::GETCOLUMNDEFINITION
;
; PURPOSE:
;       Get the description of an column from SDDS table.
;
; CATEGORY:
;       SDDS interface.
;
; CALLING SEQUENCE:
;       Result = getColumnDefinition(name)
;
; INPUTS:
;	aDefinition structure to be filled in by function
;
; OUTPUTS:
;       This function returns in a SDDS column  definition
;
;
; EXAMPLE:
;	aDefinition={columnDefinition,NAME:'',SYMBOL:'',UNITS:'',DESCRIPTION:'', $
;		     FORMAT_STRING:'',TYPE:'',FIELD_LENGTH:0l}
;       Result= getColumnDefinition('aColumnName')
;
; MODIFICATION HISTORY:
;       Written by:     Robert Daly April 27,1999
;-
	errMsg=bytarr(120)
	type=0l
	field_length=0l
	status=0l
	aDefinition={NAME:'',SYMBOL:'',UNITS:'',DESCRIPTION:'', $
		     FORMAT_STRING:'',TYPE:'',FIELD_LENGTH:0l}
	
	columnDefinition=call_external('libsddsIDL.so', $
					'getColumnDefinition',self.fd,name,type,field_length,status,errMsg,/S_VALUE)

	if(status EQ 0) then MESSAGE, 'getColumnDefinition for ' + name + ' ' + string(errMsg)
	columnArray=STR_SEP(columnDefinition,"*")
	aDefinition.NAME=columnArray(0)
	aDefinition.SYMBOL=columnArray(1)
	aDefinition.UNITS=columnArray(2)
	aDefinition.DESCRIPTION=columnArray(3)
	aDefinition.FORMAT_STRING=columnArray(4)
	case type  of
		1l:aDefinition.TYPE='DOUBLE' 
		2l:aDefinition.TYPE='FLOAT'
		3l:aDefinition.TYPE='LONG'
		4l:aDefinition.TYPE='SHORT'
		5l:aDefinition.TYPE='STRING'
		6l:aDefinition.TYPE='BYTE' 
	end
	aDefinition.FIELD_LENGTH=field_length

	return, aDefinition	
end
;*****************************************************************************
pro  sdds_table::cleanup
;+
; NAME:
;       SDDS_TABLE::INIT
;
; PURPOSE:
;       This is the cleanup code which is invoked when a  object of
;       type SDDS_TABLE is destroyed.  It cannot be called directly, but only
;       indirectly by the IDL OBJ_DESTROY() procedure.
;
; CATEGORY:
;       SDDS interface.
;
; CALLING SEQUENCE:
;       Objet_Destroy, A
;
; INPUTS:
;       A:  	The name of the SDDS_TABLE object.
;
; OUTPUTS:
;       NONE
;
;
; EXAMPLE:
;       table1 = obj_new('sdds_table', 'newTable.sdds',/new,/binary)
;       if (OBJ_VALID(table1)) then print, 'It worked!'
;	OBJ_DESTROY, table1
;
; MODIFICATION HISTORY:
;       Written by:     Robert Daly April 27,1999
;-

if(Ptr_Valid(self.parameterInfo)) then Ptr_Free, self.parameterInfo
if(Ptr_Valid(self.columnInfo)) then Ptr_Free, self.columnInfo
if(Ptr_Valid(self.arrayInfo)) then Ptr_Free, self.arrayInfo
self->sdds_table::terminate
return

end






;*****************************************************************************
function sdds_table::init, file_name, NEW=NEW, BINARY=BINARY
;+
; NAME:
;       SDDS_TABLE::INIT
;
; PURPOSE:
;       This is the initialization code which is invoked when a new object of
;       type SDDS_TABLE is created.  It cannot be called directly, but only
;       indirectly by the IDL OBJ_NEW() function.
;
; CATEGORY:
;       SDDS interface.
;
; CALLING SEQUENCE:
;       Result = OBJ_NEW('sdds_table', File_Name, /NEW,/BINARY)
;
; INPUTS:
;       File_Name:  The name of the SDDS data file.
;	NEW:	    If set create a new SDDS file.
;	BINARY:	    If data is in binary format;
;
; OUTPUTS:
;       This function returns a status to indicate whether it was able to
;       establish an sdds table associated with the specified file.
;	 This status is 1 for success, 0 for failure.  This status is
;       passed back indirectly to the routine which calls OBJ_NEW().  OBJ_NEW
;       will return a valid object pointer if this routine succeeds, and will
;       return a NULL object pointer if this routine fails.  The user should
;       test the return value of OBJ_NEW() with the IDL function OBJ_VALID().
;
;
; EXAMPLE:
;       table1 = obj_new('sdds_table', 'newTable.sdds',/new,/binary)
;       if (OBJ_VALID(table1)) then print, 'It worked!'
;
; MODIFICATION HISTORY:
;       Written by:     Robert Daly April 27,1999
;-
	status=0
	errMsg=bytarr(120)
	number=0l
	units=''
	
	if keyword_set(BINARY) then begin
		dataMode="BINARY"
	endif else begin
		dataMode="ASCII"
	end
	if keyword_set(NEW) then begin
		mode="WRITE"
	endif else begin
		mode="READ"
	end
	self.fd=call_external('libsddsIDL.so','sddsOpen',file_name, $
			mode, dataMode, status,errMsg)
 
	if(status EQ 0) then begin
		print, "sddsOpen: ",mode, dataMode, file_name, status, string(errMsg)
		 return, 0
	endif
	if(NOT(keyword_set(NEW))) then begin
		columnInfo=call_external('libsddsIDL.so', $
					'getNames',self.fd,number,"column",status,errMsg,/S_VALUE)

		if(status EQ 0) then begin
			print, "getnames:columns: ",mode, dataMode, file_name, status, string(errMsg)
			 return, 0
		endif
		self.columns=number
		if number GT 0 then begin
			self.columnInfo=PTR_NEW(STR_SEP(columnInfo,"*"))
		end
		parameterInfo=call_external('libsddsIDL.so', $
					'getNames',self.fd,number,"parameter",status,errMsg,/S_VALUE)

		if(status EQ 0) then begin
			print, "getnames:parameters: ",mode, dataMode, file_name, status, string(errMsg)
			 return, 0
		endif
		self.parameters=number
		if number GT 0 then begin
			self.parameterInfo=PTR_NEW(STR_SEP(parameterInfo,"*"))
		end
		arrayInfo=call_external('libsddsIDL.so', $
					'getNames',self.fd,number,"array",status,errMsg,/S_VALUE)
		if(status EQ 0) then begin
			print, "getnames:parameters: ",mode, dataMode, file_name, status, string(errMsg)
			 return, 0
		endif
		self.arrays=number
		if number GT 0 then begin
			self.arrayInfo=PTR_NEW(STR_SEP(arrayInfo,"*"))
		end

	endif
	self.free=1
	self.mode=mode
	self.dataMode=dataMode
	return, 1	
end
;*****************************************************************************
pro sdds_table__define, filename, NEW=NEW, BINARY=BINARY
;+
; NAME:
;       SDDS_TABLE__DEFINE
;
; PURPOSE:
;       This is the definition code which is invoked when a new object of
;       type SDDS_TABLE is created.  It cannot be called directly, but only
;       indirectly by the IDL OBJ_NEW() function,
;
; CATEGORY:
;       SDDS interface.
;
; CALLING SEQUENCE:
;       Result = OBJ_NEW('sdds_table', File_Name, /NEW)
;
; INPUTS:
;       File_Name:  The name of the SDDS file.
;	NEW: Define for writing to new table 
;
; OUTPUTS:
;       None 
;
; RESTRICTIONS:
;       This routine cannot be called directly.  It is called indirectly when
;       creating a new object of class SDDS  by the IDL OBJ_NEW()
;       function.
;
; EXAMPLE:
;       table1 = obj_new('sdds_table', 'newTable.sdds',/new,/binary)
;       if (OBJ_VALID(table1)) then print, 'It worked!'
;
; MODIFICATION HISTORY:
;       Written by:     Robert Daly
;-
    sdds_table = { 	sdds_table, $
			free: 0l, $
			file_name: '', $
			mode: '', $
			dataMode: '', $
			fd: -1l, $
			layoutWritten: 0l, $
			pageStarted: 0l, $
			parameters: 0l, $
			parameterInfo: PTR_NEW(), $
			columns: 0l, $
			rows: 0l, $
			columnInfo: PTR_NEW(), $
			arrays: 0l, $
			arrayInfo: PTR_NEW() $
		}
end
